require 'socket'
local url = require 'socket.url'

local async = require "asyncsocket"

local pushLineData = function(self, data)
  -- rather inefficient, but this is not production quality code, so who cares
  local data = self._databuf .. data
  
  while true do
    local eol = string.find(data, "\r\n", 1, true)

    if not eol then
      self._databuf = data
      return
    end

    local line = string.sub(data, 1, eol - 1)
    data = string.sub(data, eol + 2)
    self._databuf = data

    self:_processLine(line)
  end
end

local getContentSize = function(self)
  for k, v in ipairs(self._responseHeaders) do
    if string.lower(k) == "content-size" then
      return tonumber(v)
    end
  end

  return nil
end

local getLoadEventData = function(self)
  local len = getContentSize(self)
  return {
    lengthComputable = (len ~= nil),
    loaded = 0,
    total = len or 0
  }
end


local readChunkLen = function(data, start)
  local endOfNumber = string.find(data, "\r\n", start)
  if not endOfNumber then
    return nil, start
  else
    local numPart = string.sub(data, start, endOfNumber - 1)
    if string.match(numPart, "^[a-fA-F0-9]+$") == nil then
      -- TODO handle error
      return
    end
    local size = tonumber(string.sub(data, 1, endOfNumber - 1), 16)
    if size == 0 then
      return 0, nil
    end

    return size, endOfNumber + 2
  end
end


local chunkedDone = function(self)
  for k, v in pairs(self._responseHeaders) do
    if string.lower(k) == "transfer-encoding" then
      if string.lower(v) == "chunked" then
        local chunkStart = 1
        while true do
          local len
          len, chunkStart = readChunkLen(self._databuf, chunkStart)


          if len == 0 then
            return true
          elseif len == nil then
            return false
          end

          chunkStart = chunkStart + 4 + len
        end
      end
    end
  end
end


local isDataComplete = function(self)
  local len = getContentSize(self)
  if len then
    return #self._databuf >= len
  else
    return chunkedDone(self)
  end
end


local pushDataBody = function(self, data)
  self._databuf = self._databuf .. data
  if self.progress then
    self.progress(getLoadEventData(self))
  end

  if isDataComplete(self) then
    self._sock:close()
  end
end

local readHeader = function(self, line)
  if line == '' then
    self._pushdata = pushDataBody
    self.readyState = "loading"
    if self.loadstart then
      self.loadstart(getLoadEventData(self))
    end
    pushDataBody(self, "")
  else
    local key, value = string.match(line, "^([^:]+): +(.*)$")
    
    if key then
      if self._responseHeaders[key] then
        value = self._responseHeaders[key] .. ", " .. value
      end

      self._responseHeaders[key] = value

    end
  end
end

local readStatusLine = function(self, line)
  local protocol, status, text = string.match(line, "^([^ ]+) +(%d%d%d) +(.*)$")
  self.status = {protocol = protocol, status = tonumber(status), text = text}
  self._processLine = readHeader
end


local AsyncHttpRequestMt = {
  open = function(self, method, url)
    self._method = method
    self._url = url
    self._requestHeaders = {}
    self.readyState = "opened"
  end,

  setRequestHeader = function(self, header, value)
    table.insert(self._requestHeaders, {header, value})
  end,

  _error = function(self, msg)
    if self.onerror then
      self.onerror(msg)
    else
      error(msg)
    end
  end,

  _closed = function(self)
    self.readyState = "done"
    self.response = self._databuf
    if self.load then
      if self.status then
        self.load()
      end
    end
  end,

  abort = function(self)
    self._sock:close()
  end,

  send = function(self, data)
    local url_parts = url.parse(self._url, {port = "80", path = "/"})

    if not url_parts.host then
      self:_error("Host missing in URL. This is not supported.")
      return
    end

    local isPost = data ~= nil and (self._method == "POST" or self._method == "PUT")
    
    self._pushdata = pushLineData
    self._processLine = readStatusLine
    self._sock.onconnect = function()
      self._sock.onclose = function()
        self:_closed()
      end
      self._sock.ondata = function(data)
        self:_pushdata(data)
      end
      local query_and_param = ""
      if url_parts.params then
        query_and_param = query_and_param .. ";" .. url_parts.params
      end

      if url_parts.query then
        query_and_param = query_and_param .. "?" .. url_parts.query
      end

      self._sock:send(string.format("%s %s%s HTTP/1.0\r\n", self._method, url_parts.path, query_and_param))
      self._sock:send(string.format("Host: %s\r\n", url_parts.host))
      if isPost then
        self._sock:send(string.format("Content-Type: %s\r\n", data.contentType))
        self._sock:send(string.format("Content-Length: %d\r\n", #data.data))
      end
      for i, h in ipairs(self._requestHeaders) do
        self._sock:send(string.format("%s: %s\r\n", h[1], h[2]))
      end
      self._sock:send("\r\n")

      if isPost then
        self._sock:send(data.data)
        self._sock:send("\r\n")
      end
    end

    self._sock:connect(url_parts.host, tonumber(url_parts.port))
  end
}
AsyncHttpRequestMt.__index = AsyncHttpRequestMt

async.HttpRequest = function()
  return setmetatable({
    _databuf = "",
    _sock = async.socket(),
    _responseHeaders = {},
    readyState = "unsent"
  }, AsyncHttpRequestMt)
end

async.FormData = function()

end

return async
