socket = require 'socket'

local AsyncSockMt = {
  connect = function(self, addr, port) 
    return self._sock:connect(addr, port)
  end,

  send = function(self, data)
    if self._isconnected then
      return self._sock:send(data)
    else
      table.insert(self._sendbuf, data)
    end
  end,

  close = function(self)
    return self._sock:close()
  end,

  _received = function(self, buf)
    if buf and #buf > 0 then
      if self.ondata then
        self.ondata(buf)
      end
    end
  end,

  _connected = function(self)
    self._isconnected = true
    if self.onconnect then
      self.onconnect()
    end
  end,

  _onclosed = function(self)
    if self.onclose then
      self.onclose()
    end
  end,

  pump = function(self)
    if self._closed then return false end

    while true do
      local r, w, e = socket.select({self._sock}, {self._sock}, 0)

      local done = false
      if w[1] then
        if not self._isconnected then
          self:_connected()
          done = true
        end

        local count = #self._sendbuf
        done = done or (count > 0)
        for i = 1, count do
          self._sock:send(self._sendbuf[i])
          self._sendbuf[i] = nil
        end
      end
      
      if r[1] then
        done = true
        local m, e, p = self._sock:receive('*a')
        if e == "timeout" then
          self:_received(p)
        elseif e == "closed" then
          self._closed = true
          self:_received(p)
          self:_onclosed()
          return false
        else
          self:_received(m)
        end
      end

      if not done then
        return true
      end
    end
  end
}
AsyncSockMt.__index = AsyncSockMt

local sockets = {}

return {
  socket = function()
    local sock = {
      _sock = socket.tcp(),
      _buf = {},
      _isconnected = false,
      _sendbuf = {}
    }
    sock._sock:settimeout(0)
    setmetatable(sock, AsyncSockMt)
    table.insert(sockets, sock)
    return sock
  end,

  update = function()
    for i = #sockets, 1, -1 do
      if not sockets[i]:pump() then
        sockets[i] = sockets[#sockets]
        sockets[#sockets] = nil
      end
    end
  end
}
