local json = require "json.json"

Highscores = Object:extend()

function Highscores:new()
  self.scores = {}
  if love.filesystem.exists("highscores") then
    for line in love.filesystem.lines("highscores") do
      local p = string.find(line, ':')
      if p then
        local n = string.sub(line, 1, p-1)
        local t = tonumber(string.sub(line, p+1))
        self.scores[n] = t
      end
    end
  end
end


function Highscores:save()
  local s = ""
  for k, v in pairs(self.scores) do
    s = s .. string.format("%s:%.5f\n", k, v)
  end

  love.filesystem.write('highscores', s)
end


function Highscores:get(name)
  if self.scores[name] then
    return self.scores[name]
  end

  return math.huge
end


function Highscores:set(name, v)
  self.scores[name] = v
  self:save()
end


local highscoreServer = "http://www.base13.de:7477"
function Highscores:loadFromServer(trackFilename, onload)
  if self.highscoreRequest then
    self.highscoreRequest:abort()
    self.highscoreRequest = nil
  end
  local highscoreRequest = async.HttpRequest()
  highscoreRequest:open("GET", highscoreServer .. "/?track=" .. string.sub(trackFilename, 1, -5))
  highscoreRequest.load = function()
    if highscoreRequest.status.status ~= 200 then
      return
    end
    local data = highscoreRequest.response
    if not data then
      return
    end
    local decoded = json.decode(data)
    for i, v in ipairs(decoded.times) do
      if not v.name or not v.time then
        return
      end

      v.time = tonumber(v.time)
    end

    onload(decoded)
  end
  highscoreRequest:send()
  self.highscoreRequest = highscoreRequest
end

local function encodeReplay(replay)
  local e_replay = {}
  local dispatch = {
    dispatch_press = {
      up = "U",
      left = "L",
      down = "D",
      right = "R",
      x = "X",
      c = "C"
    },

    dispatch_release = {
      up = "u",
      left = "l",
      down = "d",
      right = "r",
      x = "x",
      c = "c"
    }
  }

  for i = 1, #replay, 3 do
    if dispatch[replay[i+1]] and dispatch[replay[i+1]][replay[i+2]] then
      table.insert(e_replay, replay[i] - ((i == 1) and 0 or replay[i-3]))
      table.insert(e_replay, dispatch[replay[i+1]][replay[i+2]])
    end
  end

  return e_replay
end

function Highscores:postToServer(trackFilename, playerName, time, replay, onload)
  if self.highscoreRequest then
    self.highscoreRequest:abort()
    self.highscoreRequest = nil
  end

  local highscoreRequest = async.HttpRequest()
  highscoreRequest:open("POST", highscoreServer .. "/?track=" .. string.sub(trackFilename, 1, -5))
  highscoreRequest.load = function()
    if highscoreRequest.status.status ~= 200 then
      return
    end
    local data = highscoreRequest.response
    if not data then
      return
    end
    local decoded = json.decode(data)
    for i, v in ipairs(decoded.times) do
      if not v.name or not v.time then
        return
      end

      v.time = tonumber(v.time)
    end

    onload(decoded)
  end

  local encodedReplay = encodeReplay(replay)

  highscoreRequest:send({
    data = json.encode({
      name = string.sub(playerName, 1, 20),
      time = time,
      replay = encodedReplay
    }),
    contentType = "application/json"
  })
  self.highscoreRequest = highscoreRequest

end
