lust = require "lust"

require "player"
require "ingame_state"
require "race_over_state"
require "particles"
require "highscore"
require "mainmenu"
require "socket.http"
coil = require "coil.coil"
json = require "json.json"

--local async = require "asyncsocket"
async = require "asynchttprequest"
--local AsyncHttpRequest = require "asynchttprequest"


meter = 90


function read_tracks()
  local files = love.filesystem.getDirectoryItems("data/tracks")
  table.sort(files)
  G.tracks = {}

  for _, f in ipairs(files) do
    if string.sub(f, -4) == '.lua' then
      local track = love.filesystem.load("data/tracks/" .. f)()
      G.tracks[#G.tracks+1] = {
        idx = #G.tracks+1,
        track = track,
        filename = f
      }
    end
  end
end


function love.load(args)
  -- this is only for replay verification on the highscore server, just ignore it for the game itself :-p
  G.musicVolume = 0.2
  G.flux = flux.group()

  local argc = #args
  local arg = 1
  while arg <= argc do
    if args[arg] == '--verify' then
      G.headlessTrack = args[arg+1]
      arg = arg + 1
      local headlessReplay = io.read('*a')
      G.headlessReplay = json.decode(headlessReplay)
    end

    arg = arg + 1
  end
  

  if not love.window.getPixelScale then
    love.window.getPixelScale = love.window.getDPIScale
  end

  local a = love.getVersion()
  if a >= 11 then
    local setColor = love.graphics.setColor
    print("polyfill")
    love.graphics.setColor = function(r,g,b,a) setColor((r or 255)/255, (g or 255)/255, (b or 255)/255, (a or 255)/255) end

    local clear = love.graphics.clear
    love.graphics.clear = function(r,g,b,a) clear((r or 255)/255, (g or 255)/255, (b or 255)/255, (a or 255)/255) end


    love.filesystem.exists = love.filesystem.getInfo
  end

	love.physics.setMeter(meter)
  G.playerName = "Sir Race-A-Lot"
  G.density = 1

  if not G.headless then
    if love.filesystem.exists("playername") then
      local file = love.filesystem.read("playername")
      local eol = string.find(file, '[\r\n]')
      if eol then
        file = string.sub(file, 1, eol - 1)
      end
      G.playerName = file
    end
    G.density = love.window.getPixelScale( ) * love.window.getPixelScale()

    G.highscores = Highscores()
    G.highscores:save()

    G.sounds = {}
    for i, n in ipairs({
      "countdown_short",
      "countdown_long",
      "car_fullspeed",
      "turn_car",
      "shift",
      "finish",
      "explosion",
      "thud",
      "brake"
    }) do
      G.sounds[n] = love.audio.newSource("data/audio/" .. n .. ".ogg", "static")
    end 

  if love.system.getOS() == 'Web' then
      G.engine = {
        [1] = love.audio.newSource("data/audio/car_0_1.ogg", "static"),
        [2] = love.audio.newSource("data/audio/car_0_2.ogg", "static"),
        [3] = love.audio.newSource("data/audio/car_0_3.ogg", "static"),
        [4] = love.audio.newSource("data/audio/car_0_4.ogg", "static"),
        [5] = love.audio.newSource("data/audio/car_0_5.ogg", "static"),
        [6] = love.audio.newSource("data/audio/car_0_6.ogg", "static"),
        [7] = love.audio.newSource("data/audio/car_0_7.ogg", "static"),
        [8] = love.audio.newSource("data/audio/car_0_7.ogg", "static"),
        [9] = love.audio.newSource("data/audio/car_0_8.ogg", "static"),
        [10] = love.audio.newSource("data/audio/car_fullspeed.ogg", "static"),
        [11] = love.audio.newSource("data/audio/car_1_1.ogg", "static")
      }

      G.sounds.car_fullspeed = {
        pitch = 10,
        volume = 1.0,

        stop = function(self)
          for k, v in ipairs(G.engine) do
            v:stop()
          end
        end,

        play = function(self)
          for k, v in ipairs(G.engine) do
            v:play()
          end
        end,

        setPitch = function(self, pitch)
          self.pitch = math.max(1, math.min(11, math.floor(10*pitch)))
          for k, v in ipairs(G.engine) do
            if k ~= self.pitch then
              v:setVolume(0)
            else
              v:setVolume(self.volume)
            end
          end
        end,

        setVolume = function(self, vol)
          self.volume = vol
          for k, v in ipairs(G.engine) do
            v:setVolume(0)
          end
          G.engine[self.pitch]:setVolume(self.volume)
        end,

        setLooping = function(self, l)
          for k, v in ipairs(G.engine) do
            v:setLooping(l)
          end
        end

      }
    end

    --G.sounds.car_fullspeed = G.sounds.car_slow
    G.sounds.car_fullspeed:setLooping(true)




    G.tinyFont = love.graphics.newFont(22)
    G.smallFont = love.graphics.newFont("data/coolvetica.ttf", 32)
    G.largeFont = love.graphics.newFont("data/coolvetica.ttf", 64)
  end
  G.keyconfig = lust.KeyConfig(
    {"throttle", "Throttle", "up"},
    {"break", "Brake", "down"},
    {"up", "Pitch Up", "left"},
    {"down", "Pitch Down", "right"},
    {"shift", "Shapeshift", "x"},
    {"spin", "Spin Vehicle", "c"},
    {"reset", "Reset Game", "r"}
  )
	G.core = lust.Core()

  Particle.load()
  Track.load()
  Player.load()
  if not G.headless then
    G.music = love.audio.newSource("data/music.ogg", "stream")
    G.music:setLooping(true)
    G.music:setVolume(0.2)
    G.music:play()
    RaceOverState.load()

    read_tracks()
    G.core.states:push(MainMenu())
  else
    local trackdata = love.filesystem.load("data/tracks/" .. G.headlessTrack .. ".lua")()
    local newIngame = IngameState({track = trackdata}, G.headlessReplay)
    G.core.states:push(newIngame)
--    G.core.states:push(CountdownState(newIngame))
    G.core.states:update(100)
    love.event.quit()
  end
end


function love.draw()
  if not G.headless then
    G.core.states:draw()
  end
end


function music(onoff)
  G.flux:to(G, 5, {musicVolume = onoff and 1.0 or 0.2}):onupdate(function() G.music:setVolume(G.musicVolume) end)
end


function love.update(dt)
  G.flux:update(dt)
  if not G.headless then
    async:update()
  end

	if not G.core.states:update(dt) then
    love.event.quit()
  end
end
