MainMenu =  lust.Gamestate:extend()

OnlineHighscores = require "onlinehighscore"

function MainMenu:new()
	self.input = lust.BoundInput()
	self.input:bind_key("up", function() self:menu_up() end)
	self.input:bind_key("down", function() self:menu_down() end)
	self.input:bind_key("left", function() self:menu_left() end)
	self.input:bind_key("right", function() self:menu_right() end)
  self.input:bind_key("return", function() self:menu_enter() end)
	self.input.action_map_func = function(key) return key end
  self.track = 1
  self.menu = 1

  self.nameInput = lust.BoundInput()
  self.nameInput:bind_key("escape", function()
    G.playerName = self.oldName
    G.core.input:pop()
    love.keyboard.setTextInput(false)
    self.enteringName = false
  end)
  self.nameInput:bind_key("return", function() 
    if #G.playerName > 0 then
      love.filesystem.write("playername", G.playerName)
      G.core.input:pop()
      self.enteringName = false
      love.keyboard.setTextInput(false)
    end
  end)
  self.nameInput:bind_key("backspace", function()
    G.playerName = string.sub(G.playerName, 1, -2)
  end)


  self.onlineHighscores = OnlineHighscores(290)

  self:loadHighscore()
end


function MainMenu:loadHighscore()
  self.onlineHighscores:load(G.tracks[self.track])
end


function MainMenu:enter()
  G.core.input:push(self.input)
end

function MainMenu:leave()
  G.core.input:pop()

end


function MainMenu:update(dt)
  self.onlineHighscores:update(dt)
end

function MainMenu:draw()
  love.graphics.setFont(G.largeFont)
  local w, h = love.graphics.getDimensions()
  love.graphics.setColor(255,0,0)
  love.graphics.printf("Turbo/Glide", 0, 10, w, 'center')
  love.graphics.setFont(G.smallFont)
  love.graphics.printf("made in 48h for Ludum Dare 35 by rnlf", 0, 80, w, 'center')
  love.graphics.setColor(255,255,255)
  love.graphics.printf("<", 300, 220, 50, 'center')
  love.graphics.printf(">", w-350, 220, 50, 'center')

  love.graphics.setFont(G.largeFont)
  love.graphics.printf(string.format("Track #%d", self.track), 50, 170, w-100, 'center')
  love.graphics.setFont(G.smallFont)
  local record = G.highscores:get(G.tracks[self.track].filename)
  local records
  if record ~= math.huge then
    local m = math.floor(record / 60)
    local s = math.fmod(record, 60)
    records = string.format("%02d:%05.2f", m, s)
  else
    records = "???"
  end
  love.graphics.printf(string.format("Your Personal Best: %s", records), 50, 250, w-100, 'center')

  if self.menu == 1 then
    love.graphics.setColor(255,255,255,255)
  else
    love.graphics.setColor(255,255,255,64)
  end
  love.graphics.printf("Start Race!", 0, 620, w, 'center')
  if self.menu == 2 then
    love.graphics.setColor(255,255,255,255)
  else
    love.graphics.setColor(255,255,255,64)
  end
  love.graphics.printf("Player Name: " .. G.playerName .. (self.enteringName and '_' or ''), 0, 650, w, 'center')
  if self.menu == 3 then
    love.graphics.setColor(255,255,255,255)
  else
    love.graphics.setColor(255,255,255,64)
  end
  love.graphics.printf("Exit Game", 0, 680, w, 'center')

  -- highscores
  self.onlineHighscores:draw()
  
  love.graphics.setFont(G.smallFont)
  love.graphics.setColor(255,255,255)
  if string.lower(G.playerName) == "sir race-a-lot" then
    love.graphics.printf("WARNING: Highscore server ignores times with this name!", 0, h - G.smallFont:getHeight(), w, 'center')
  end

  if #G.playerName < 3 then
    love.graphics.printf("Name must be at least three characters", 0, h - G.smallFont:getHeight(), w, 'center')
  end
end



function MainMenu:menu_left()
  self.track = self.track - 1
  if self.track == 0 then
    self.track = #G.tracks
  end
  self:loadHighscore()
end

function MainMenu:menu_right()
  self.track = self.track % #G.tracks + 1
  self:loadHighscore()
end

function MainMenu:menu_up()
  self.menu = self.menu - 1
  if self.menu == 0 then
    self.menu = 3
  end
end

function MainMenu:menu_down()
  self.menu = self.menu % 3 + 1
end

function MainMenu:menu_enter()
  if self.menu == 1 then
    startRace(G.tracks[self.track])
  elseif self.menu == 2 then
    self.oldName = G.playerName
    if G.playerName == "Sir Race-A-Lot" then
      G.playerName = ""
    end
    G.core.input:push(self.nameInput)
    love.textinput = function(text)
      G.playerName = string.sub(G.playerName .. text, 1, 20)
    end
    love.keyboard.setTextInput(true)
    self.enteringName = true
  else
    G.core.states:pop()
  end
end
