local OnlineHighscores = Object:extend()

function OnlineHighscores:new(drawY)

  self.drawY = drawY
  self.flux = flux.group()

  self.fluxes = {}
  self.highscores = {}
  self.highscoreOffset = {}
  for i = 1, 10 do
    self.highscoreOffset[i] = 1
  end
  self.t = 0

end


function OnlineHighscores:draw()
  love.graphics.setFont(G.tinyFont)
  local lh = G.tinyFont:getHeight()
  local w, h = love.graphics.getDimensions()

  if self.drawBackground then
    love.graphics.setColor(0,0,0,100)
    for i = 1, math.min(10, #self.highscores) do
      love.graphics.rectangle('fill', ((i % 2 == 1) and 1 or -1) * self.highscoreOffset[i] * w, self.drawY + lh*i, w, lh)
    end
  end

  for i = 1, math.min(10, #self.highscores) do
    local time = self.highscores[i].time
    local m = math.floor(time / 6000)
    local s = math.floor(math.floor(time % 6000) / 100)
    local hs = time % 100
    local ts = string.format("%02d:%02d.%02d", m, s, hs)
    if self.player and self.time then
      if self.player == self.highscores[i].name and self.time == self.highscores[i].time then
        local gb = math.sin(self.t * 10) * 128 + 128
        love.graphics.setColor(255, gb, gb, 255)
      else
        love.graphics.setColor(255,255,255,(1-self.highscoreOffset[i]) * 255)
      end
    else
      love.graphics.setColor(255,255,255,(1-self.highscoreOffset[i]) * 255)
    end
    love.graphics.printf(self.highscores[i].name, -self.highscoreOffset[i] * w / 2, self.drawY + lh*i, w / 2 - 20, 'right')
    love.graphics.printf(ts, w/2+20+self.highscoreOffset[i] * w / 2, self.drawY + lh*i, w / 2, 'left')
  end
end

function OnlineHighscores:update(dt)
  self.flux:update(dt)
  self.t = self.t + dt
end

function OnlineHighscores:load(track, player, time, replay)
  self.player = player
  self.time = time
  self.highscores = {}
  local ondone = function(data)
    self.highscores = data.times
    for i = 1, 10 do
      if data[i] then
        local t = data[i].time
        data[i].time = string.format("%02d:%02d.%02d", math.floor(t / 6000), math.floor((t % 6000) / 100), t % 100)
      end

      if self.fluxes[i] then
        self.fluxes[i]:stop()
      end
      self.highscoreOffset[i] = 1
      self.fluxes[i] = self.flux:to(self.highscoreOffset, 0.5, {[i] = 0}):delay(i*0.1)
    end
  end

  if player and time then
    G.highscores:postToServer(track.filename, player, time, replay, ondone)
  else
    G.highscores:loadFromServer(track.filename, ondone)
  end

end

return OnlineHighscores
