Player = Object:extend()

local angleStep = 10 
local angleSteps = 360 / angleStep
local playerSpriteSize = 128
local playerOffsetX = 65 --/ playerSpriteSize
local playerOffsetY = 63 --/ playerSpriteSize

function Player.sprite(filename, w, h, anims)

  local image
  if not G.headless then
    image = G.core.images:load(filename)
  end
  if not Player.anims then
    Player.anims = {}
  end 
  local iw, ih
  if not G.headless then
    iw, ih = image:getDimensions()
  else
    iw, ih = 1
  end

  local fpr = math.floor(iw / w)

  for _, a in ipairs(anims) do
    local f = {}
    for i, cf in ipairs(a[2]) do
      local x = cf % fpr 
      local y = math.floor(cf / fpr)
      if not G.headless then
        f[i] = G.core.quads:make(x * w, y * h, w, h, iw, ih) 
      else
        f[i] = i
      end
    end 

    Player.anims[a[1]] = { 
      image = image,
      frames = f,
      fps = a[3]
    }   
  end 
end


function Player.load()
	local transToPlane = {}
	local transToCar = {}
	local spinPlane = {}
	local spinCar = {}

	for i = 24, 29 do
		table.insert(spinCar, i)
	end
	for i = 30, 34 do
		table.insert(spinPlane, i)
	end
	for i = 0, 23, 1 do
		table.insert(transToPlane, i)
		table.insert(transToCar, 23-i)
	end
	for d = 0, 360-angleStep, angleStep do
		local filename = string.format("data/car-%03d.png",d)
		Player.sprite(filename, 128, 128, {
			{string.format("toPlane%03d", d), transToPlane,  48},   -- 1/2s == 50ticks
			{string.format("toCar%03d", d), transToCar,  48},       -- 1/2s == 50ticks
			{string.format("car%03d", d), {0}, 1},
			{string.format("plane%03d", d), {23}, 1},
			{string.format("spinCar%03d", d), spinCar, 24},         -- 1/4s == 25ticks
			{string.format("spinPlane%03d", d), spinPlane, 25},     -- 1/5s == 20ticks
		})
	end
	
end


function Player:play(anim, reset, onDone)
	if reset then
		self.curframe = 0
	end
	self.animName = anim

  local anim = self:getAnim(0)

	self.onDone = onDone

	if self.curframe >= #anim.frames then
		self.curframe = math.fmod(self.curframe, #anim.frames)
	end

end


function Player:doneAnim()
	if self.onDone then
    local f = self.onDone
		self.onDone = nil
		f()
	end
end

function Player:getAnim(lerp)
--	local dir = -self.bodies.chassis:getAngle()
  local dir = -lust.tools.lerp(self.lastA, self.angle, lerp)
	
	if self.form == 'car' or self.form == 'spinCar' or self.form == "toPlane" then
			dir = dir*self.scale
  elseif self.form == 'plane' or self.form == 'spinPlane' or self.form == "toCar" then
		if self.scale > 0 then
			dir = dir + math.pi
		elseif self.scale < 0 then
			dir =  -dir

		end
	end

	while dir < 0 do
		dir = dir + 2 * math.pi
	end
	while dir >= 2*math.pi do
		dir = dir - 2 * math.pi
	end
	local d = math.deg(dir)
	local angle = d / angleStep
	if angle > angleSteps - 0.5 then
		angle = 0
	else
		
		local a, b = math.modf(angle)
		if b > 0.5 then
			angle = a + 1
		else
			angle = a
		end
	end

	angle = angle * angleStep

  local animName = string.format("%s%03d", self.animName, angle)

  return Player.anims[animName], d - angle
end


function Player:shape()
	if self.form == 'car' or self.form == 'toPlane' or self.form == 'spinCar' then
		return 'car'
	else
		return 'plane'
	end
end

function Player:getAfFixture()
		return self.fixtures.airframe, self.bodies.airframe
end


function Player:kill()
  if not G.headless then
    G.sounds.car_fullspeed:stop()
    G.sounds.explosion:play()
  end
  self.dead = true
end


function Player:die()
  if not G.headless then
    local count = 18 
    local x,y = self:getPos()
    local spawn100 = function()
      if count % 2 == 0 then
        for i = 1, 50 do
          local a = love.math.random() * 2*math.pi
          local v = 500 + love.math.random() * 3000
          local s = 10 + love.math.random() * 20

          local g = love.math.random() * 255
          local bl = love.math.random()

          local p = self.state:spawnParticle(x, y, s, {255*bl,g*bl,10}, 1+love.math.random()*3, {math.sin(a) * v, math.cos(a)*v}, 0, 0.1)
          p.body:setLinearDamping(3)
          p.fixture:setRestitution(0.9)
          p.dsts = 40 + love.math.random() * 20
          if bl > 0.3 then
            p.drawMode = 'add'
          end
        end
      end

      count = count - 1
      if count == 0 then
        self.state.preFrame = nil
      end
    end
    self.state.preFrame = spawn100
    spawn100()
  end


  for _, b in pairs(self.bodies) do
    b:destroy()
  end
end



function Player:new(state)
  self.rpm = 0
	self.pitch = 0
	self.form = 'car'
	self.state = state

	self.scale = -1

	self.curframe = 0
	self.animName = 'car'
	self.bodies = {
		chassis = love.physics.newBody(state.world, 200, 200, 'dynamic'),
		back_wheel = love.physics.newBody(state.world, 160, 200, 'dynamic'),
		front_wheel = love.physics.newBody(state.world, 240, 200, 'dynamic'),
		airframe = love.physics.newBody(state.world, 210, 225, 'dynamic'),
    bumper = love.physics.newBody(state.world, 200, 200, 'dynamic'),
		airframe2 = love.physics.newBody(state.world, 190, 225, 'dynamic')
	}

	self.bodies.back_wheel:setMass(0.001)
	self.bodies.front_wheel:setMass(0.001)
  self.bodies.bumper:setMass(0.00001)

	self.bodies.chassis:setAngularDamping(10)
	self.bodies.chassis:setLinearDamping(0.1)
	
	self.fixtures= {
		back_wheel = love.physics.newFixture(self.bodies.back_wheel, love.physics.newCircleShape(0, 0, 20), 200 / G.density),
		front_wheel = love.physics.newFixture(self.bodies.front_wheel, love.physics.newCircleShape(0, 0, 20), 200 / G.density),
		chassis = love.physics.newFixture(self.bodies.chassis, love.physics.newRectangleShape(0, -25, 120, 25),1600 / G.density),
		airframe = love.physics.newFixture(self.bodies.airframe, love.physics.newRectangleShape(0,-25,100,30), 0.001 / G.density),
		airframe2 = love.physics.newFixture(self.bodies.airframe2, love.physics.newRectangleShape(10,-60,100,20), 0.001 / G.density),
	}
	self.bodies.chassis:setMass(1000)

  self.fixtures.chassis:setCategory(3)
	self.fixtures.back_wheel:setCategory(2)
	self.fixtures.front_wheel:setCategory(2)
	self.fixtures.airframe:setSensor(true)
	self.fixtures.airframe2:setSensor(true)

  self.fixtures.chassis:setMask(2, 5)
  self.fixtures.front_wheel:setMask(2, 3, 5)
  self.fixtures.back_wheel:setMask(2, 3, 5)

	self.motorJoint = love.physics.newRevoluteJoint(self.bodies.back_wheel, self.bodies.chassis, 160, 200, false)
	love.physics.newRevoluteJoint(self.bodies.front_wheel, self.bodies.chassis, 240, 200, false)
	love.physics.newWeldJoint(self.bodies.chassis, self.bodies.airframe, 200, 150, false)
	love.physics.newWeldJoint(self.bodies.chassis, self.bodies.airframe2, 200, 150, false)
	self.fixtures.back_wheel:setFriction(1)
	self.fixtures.front_wheel:setFriction(1)


  self:warpTo(200,200)
  self.lastA = 0
  self.angle = 0
	self:play("car")
end

function Player:updateGfx(dt)
  self:updateEngineSoundAndParticles(dt)

end


function Player:warpTo(x, y)
	local px, py = self:getPos()
	local dx, dy = x - px, y - py
	for _, b in pairs(self.bodies) do
		px, py = b:getWorldCenter()
		b:setPosition(px + dx, py + dy)
	end

  self.lastX = x
  self.lastY = y
  self.x = x
  self.y = y
end


function Player:updateEngineSoundAndParticles(dt)
  -- spawn exhaust and play engine sound
	if self.thrust and self:shape() == 'car' then
    if not G.headless then
      local a = self.bodies.chassis:getAngle()
      local f
      if self.scale > 0 then
        f = self.bodies.front_wheel
      else
        f = self.bodies.back_wheel
      end
      local x,y = f:getWorldCenter()

      local c = love.math.random(64, 192)
      local b = a + math.rad(love.math.random(-30, 30))
      local p = love.math.random(30, 60)
      local evx = math.cos(b)
      local evy = math.sin(b)
      local pa = self.state:spawnParticle(x, y, 5, {c,c,c}, 1,
        {-evx*p, -evy*p}, 0, -0.1)
      pa.body:setLinearDamping(0.5)
      pa.fixture:setRestitution(0.01)
      pa.dsts = love.math.random(7, 15)
    end
  end

  if G.headless then
    return
  end

  -- stop sound if necessary
  if self.dead or self:shape() == "plane" or self.state.win then
    G.sounds.car_fullspeed:stop()
  else
    G.sounds.car_fullspeed:play()
  end

  G.sounds.car_fullspeed:setPitch(0.1+math.sin(self.rpm * math.pi / 2))
  G.sounds.car_fullspeed:setVolume(0.5 * (0.4+0.4*math.sin(self.rpm * math.pi / 2)))
end


function Player:updateBehaviour(dt)


	if self.thrust and self:shape() == 'car' then
    self.rpm = math.min(self.rpm + 0.8 * dt, 1)
  else
    self.rpm = math.max(self.rpm - 0.5 * dt, 0)
  end

  if self.state.countdown then
    return
  end

  -- currently some game events depend on the animation times, so they must be here
  local anim = self:getAnim(0)
	self.curframe = self.curframe + dt * anim.fps
	if self.curframe >= #anim.frames then
		self.curframe = math.fmod(self.curframe, #anim.frames)
		self:doneAnim()
	end

	if self.thrust then
    if self:onGround() then
      local a = self.bodies.chassis:getAngle()


      local vx = math.cos(a)
      local vy = math.sin(a)
      self.bodies.chassis:applyForce(-self.scale*vx*10000, -self.scale*vy*10000)
    end

		self.bodies.back_wheel:applyTorque(-self.scale*50000000*self.rpm)
		self.bodies.front_wheel:applyTorque(-self.scale*50000000*self.rpm)
	end

	if self.brakes_on then
    self.rpm = 0
		self.bodies.back_wheel:setAngularVelocity(0)
		self.bodies.front_wheel:setAngularVelocity(0)

    if self:onGround() then
      local vx, vy = self.bodies.chassis:getLinearVelocity()
      local v = lust.tools.length(vx, vy)

      self.bodies.chassis:applyForce(-vx*2000, -vy*2000)
      

      if not G.headless and not self.screetchplayed then
        G.sounds.brake:setVolume(v / 1000)
        G.sounds.brake:play()
        self.screetchplayed = true
      end
    else
      self.screetchplayed = false
    end
	end


	local downX, downY = lust.tools.normalize(self.bodies.chassis:getWorldVector(0, self.scale*-1))
	local velX, velY = self:getVel()
	velX, velY = velX, velY
	local velSq = velX*velX + velY*velY


	if velSq > 1000*1000 then
		velX, velY = lust.tools.scale(velX, velY, 1000)
		self.bodies.chassis:setLinearVelocity(velX, velY)
		velSq = 1000
	end

	self.velA = {x = velX, y = velY}

	local vel = math.sqrt(velSq)
	local fwdX, fwdY = lust.tools.normalize(self.bodies.chassis:getWorldVector(1, -self.scale*0.1))
	local velNX, velNY = lust.tools.normalize(velX, velY)

	local aoA = (fwdX * velNX + fwdY * velNY)
	local attackA = math.acos(aoA)
	local attack = math.sin(attackA)
	local a = self.bodies.chassis:getAngle()

	local ias = math.max(0, velX * fwdX + velY * fwdY)

	self.ias = {x = fwdX * ias, y = fwdY * ias}

	local lift = ias *ias*1.5*math.max(0,math.cos(attackA * math.pi * 2))
	if lift > 150000 then
		lift = 150000
	end

	local fac = math.max(0, aoA)

	local cha = self.bodies.chassis

	local drag = velSq * 0.15 * math.max(0,math.cos(attackA * math.pi * 2))* (1 + 10 * math.abs((velNX * (-downX) + velNY * (-downY))))
	self.dragF = {
		x = -drag * velNX,
		y = -drag * velNY
	}


	if self.form == 'plane' or self.form == 'toPlane' then
		cha:applyForce(self.dragF.x, self.dragF.y)

		if ias > 50 then
			local t = 30000000
			if lust.tools.sig(self.pitch) ~= lust.tools.sig(self.scale) then
				t = t /1.5 
			end
			cha:applyTorque(t * ((self.pitch-self.scale*0.2))) -- * vel)
		end

		local a = cha:getAngle()
		local vx, vy = math.cos(a), math.sin(a)
		self.orientA = {x = vx, y = vy}


			local td = velNY * vx - velNX * vy
			cha:applyTorque(50000000 * td)
			self.liftF = {
				x = -self.scale * fwdY * lift,
				y = self.scale * fwdX * lift
			}
	
			cha:applyForce(self.liftF.x, self.liftF.y)
	end
end



function Player:update(dt)
  self:updateBehaviour(dt)

  local x, y = self:getPos()
  self.lastX, self.lastY = x, y
  self.x, self.y = x, y

  local a = self.bodies.chassis:getAngle()
  self.lastA = self.angle
  self.angle = a
end


function Player:spin()
	if self.form == 'car' then
		self.form = 'spinCar'
		self:play('spinCar', true, function() self:spinDone() end)
    if not G.headless then
      G.sounds.turn_car:play()
    end
	elseif self.form == 'plane' then
		self.form = 'spinPlane'
		self:play('spinPlane', true, function() self:spinDone() end)
  else
    self.buffer = 'spin'
	end
end


function Player:spinDone()
	self.scale = -self.scale
	if self.form == 'spinCar' then
		self.form = 'car'
		self:play('car')
	elseif self.form == 'spinPlane' then
		self.form = 'plane'
		self:play('plane')
	end

  self:runBufferedCommand()
end


function Player:runBufferedCommand()
  if self.buffer then
    self[self.buffer](self)
    self.buffer = nil
  end

end


function Player:shift()
	if self.form == 'car' or self.form == 'toCar' then
		self.form = 'toPlane'
		self:play('toPlane', true, function() self:shiftDone() end)
	elseif self.form == 'plane' or self.form == 'toPlane' then
		self.form = 'toCar'
		self:play('toCar', true, function() self:shiftDone() end)
  else
    self.buffer = 'shift'
    return
	end

  if not G.headless then
    G.sounds.shift:play()
  end
end


function Player:shiftDone()
	if self.form == 'toCar' then
		self.form = 'car'
		self:play('car')
		self.bodies.chassis:setGravityScale(1)
		self.bodies.back_wheel:setGravityScale(1)
		self.bodies.front_wheel:setGravityScale(1)
		self.bodies.chassis:setLinearDamping(0.1)
		self.bodies.chassis:setMass(1000)
		self.fixtures.front_wheel:setMask(2, 5)
		self.fixtures.back_wheel:setMask(2, 5)
    self.fixtures.chassis:setMask(5)
	else
		self.form = 'plane'
		self:play('plane')
		self.bodies.chassis:setLinearDamping(0.1)
		self.bodies.chassis:setGravityScale(0.2)
		self.bodies.back_wheel:setGravityScale(0.2)
		self.bodies.front_wheel:setGravityScale(0.2)
		self.bodies.chassis:setMass(10)
		self.fixtures.front_wheel:setMask(1, 2, 5)
		self.fixtures.back_wheel:setMask(1, 2, 5)
    self.fixtures.chassis:setMask(1, 5)
	end

	if self.scale > 0 then
		self.bodies.chassis:setAngle(self.bodies.chassis:getAngle() + math.pi)
	end

  self:runBufferedCommand()
end


function Player:throttle(onoff)
	self.thrust = onoff
end

function Player:onGround()
  local cs = self.state.world:getContactList()

  for _, c in ipairs(cs) do
    local a, b = c:getFixtures()
    if (a == self.fixtures.front_wheel or a == self.fixtures.back_wheel) and b:getUserData() == 'ground' then
      if c:isTouching() then
        return true
      end
    end

    if  (b == self.fixtures.front_wheel or b == self.fixtures.back_wheel) and a:getUserData() == 'ground' then
      if c:isTouching() then
        return true
      end
    end
  end

  return false
end

function Player:brake(onoff)
	self.brakes_on = onoff

  if not G.headless and onoff then
    self.screetchplayed = false
  end
end

function Player:setPitch(amt)
	self.pitch = self.pitch + amt
end


function Player:getPos()
	return self.bodies.chassis:getX(), self.bodies.chassis:getY()
end

function Player:getVel()
	return self.bodies.chassis:getLinearVelocity()
end


function Player:draw(lerp)
  if G.headless then
    return
  end
  love.graphics.setColor(255,255,255,255)
  local anim, a_offset = self:getAnim(lerp)
	local x = self.bodies.chassis:getX()
	local y = self.bodies.chassis:getY() * renderScale
	love.graphics.draw(anim.image, anim.frames[math.floor(self.curframe) + 1], x, y, 0, self.scale, 1, playerOffsetX, playerOffsetY)
end
