RaceOverState = lust.Gamestate:extend()

function RaceOverState.load()
  RaceOverState.canvas = love.graphics.newCanvas()
  RaceOverState.shader = love.graphics.newShader [[
    uniform float desat;
    uniform float darken;
    vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      vec4 tx = Texel(texture, texture_coords);
      float l = (tx.r + tx.g + tx.b) / 3.0;
      vec4 gray = vec4(l,l,l,1.0);
      
      //float l = color.r;
      return darken * (desat * gray + (1.0-desat) * tx);
    }
  ]]
  
  RaceOverState.shader:send("darken", 0.5)
  RaceOverState.shader:send("desat", 0.5)
end

function RaceOverState:new(ingameState)
  self.ingameState = ingameState

  self.flux = flux.group()
  self.gray = 0
  self.dark = 1
  self.input = lust.BoundInput()
  self.input:bind_key("right", function() self.menuActive = self.menuActive % #self.menu + 1 end)
  self.input:bind_key("left", function()
    self.menuActive = self.menuActive -1 
    if self.menuActive == 0 then
      self.menuActive = #self.menu
    end
  end)

  self.input:bind_key("return", function()
    self.menu[self.menuActive][2]()
  end)

  self.input.action_map_func = function(key)
    return key
  end


  if self.ingameState.win then
    self.best = G.highscores:get(self.ingameState.trackData.filename)

    if self.best > self.ingameState.time / 100 then
      self.newBest = true
      G.highscores:set(self.ingameState.trackData.filename, self.ingameState.time / 100)
    end
  end

  local time = math.floor(self.ingameState.time)
  local micro = (self.ingameState.time * 1000) % 10
  if micro >= 5 then
    time = time + 1
  end
  --[[
  G.highscores:postToServer(self.ingameState.trackData.filename, G.playerName,
    time,
    function() end)
  --]]

  self.onlineHighscores = OnlineHighscores(320)
  self.onlineHighscores.drawBackground = true
  if self.ingameState.win then
    self.onlineHighscores:load(self.ingameState.trackData, G.playerName, time, self.ingameState.replayKeys)
  else
    self.onlineHighscores:load(self.ingameState.trackData)
  end

  --[[
  for i = 1, #self.ingameState.replayKeys, 3 do
    print(self.ingameState.replayKeys[i], 
      self.ingameState.replayKeys[i+1], 
      self.ingameState.replayKeys[i+2])
  end
  --]]
end


function RaceOverState:update(dt)
  self.ingameState:update(dt)
  self.flux:update(dt)
  self.onlineHighscores:update(dt)
end


function RaceOverState:draw()
  love.graphics.reset()
  love.graphics.origin()
  love.graphics.setCanvas(RaceOverState.canvas)
  love.graphics.clear()
  self.ingameState:draw()
  love.graphics.setCanvas()
  love.graphics.reset()
  love.graphics.origin()
--  love.graphics.rectangle('fill', 0, 0, love.graphics.getDimensions())
  love.graphics.setShader(RaceOverState.shader)
  RaceOverState.shader:send("darken", self.dark)
  RaceOverState.shader:send("desat", self.gray)
  love.graphics.draw(RaceOverState.canvas)
  love.graphics.setShader()
  
  local w, h = love.graphics.getDimensions()
  if self.ingameState.win then
    love.graphics.setColor(0,0,0, 200)
    love.graphics.rectangle('fill', self.offsets.caption_bg, 80, w, G.largeFont:getHeight() + 60)
    love.graphics.rectangle('fill', self.offsets.time_bg, 230, w, G.smallFont:getHeight() + 50)
    love.graphics.setColor(255,255,255,255)
    love.graphics.setFont(G.largeFont)
    love.graphics.printf("Course Finished", self.offsets.caption, 110, w, 'center')
    love.graphics.setFont(G.smallFont)
    local ts = "Time: " .. self.ingameState:format_time()
    if self.newBest then
      ts = ts .. " NEW RECORD!"
    else
      local m = math.floor(self.best / 60)
      local s = math.fmod(self.best, 60)
      ts = ts .. string.format(" (Record: %02d:%05.2f)", m , s)
    end
    love.graphics.printf(ts, self.offsets.time, 260, w, 'center')
  else
    love.graphics.setColor(0,0,0, 200)
    love.graphics.rectangle('fill', self.offsets.caption_bg, 180, w, G.largeFont:getHeight() + 60)
    love.graphics.setColor(255,255,255,255)
    love.graphics.setFont(G.largeFont)
    love.graphics.printf("You Failed", self.offsets.caption, 210, w, 'center')
  end

  love.graphics.setColor(0,0,0, 200)
  love.graphics.rectangle('fill', self.offsets.menu_bg, h / 2 + 100 + G.smallFont:getHeight() + 120, w, G.smallFont:getHeight() + 60)

  love.graphics.setFont(G.smallFont)

  for i, m in ipairs(self.menu) do
    if i == self.menuActive then
      love.graphics.setColor(255,255,255,255)
    else
      love.graphics.setColor(255,255,255,80)
    end
    love.graphics.printf(m[1], self.offsets.menu + (i - 1) * w / #self.menu, h / 2 + 100 + G.smallFont:getHeight() + 150, w / #self.menu, 'center')
  end

  self.onlineHighscores:draw()

end


function RaceOverState:enter()
  local w, h = love.graphics.getDimensions()
  self.offsets = {
    caption_bg = w,
    time_bg = -w,
    caption = -w,
    time = w,
    menu_bg = -w,
    menu = w
  }
  self.flux:to(self.offsets, 0.2, {caption_bg = 0}):delay(0.1)
  self.flux:to(self.offsets, 0.2, {caption = 0}):delay(0.3)
  self.flux:to(self.offsets, 0.2, {time_bg = 0}):delay(0.2)
  self.flux:to(self.offsets, 0.2, {time = 0}):delay(0.4)
  self.flux:to(self.offsets, 0.2, {menu_bg = 0}):delay(0.7)
  self.flux:to(self.offsets, 0.2, {menu = 0}):delay(0.9)

  self.flux:to(self, 1, {gray = 0.8}):delay(0.5)
  self.flux:to(self, 1, {dark = 0.7}):delay(0.75)

  self.menu = {
    {"Exit", function() 
      G.core.states.stack[1]:loadHighscore()
      G.core.states:pop()
    end},
    {"Retry", function() 
  --    G.core.states:replace(IngameState(self.ingameState.trackData))
      G.core.states:pop()
      startRace(self.ingameState.trackData)
    end},
    {"Replay", function()
      local newIngame = IngameState(self.ingameState.trackData, self.ingameState.replayKeys)
      G.core.states:push(newIngame)
      G.core.states:push(CountdownState(newIngame))
    end},
  }

  self.menuActive = 2

  if self.ingameState.win then
    self.menu[#self.menu+1] = {
      "Next Course", function() 
        G.core.states:pop()
        startRace(G.tracks[self.ingameState.trackData.idx % #G.tracks + 1])
      end
    }
  end

  G.core.input:push(self.input)

end


function RaceOverState:leave()
  G.core.input:pop(self.input)

end
